// %PARAMETERS = "CH20LIST C:\UT2004"
//Identifies the package
//Stage.uc
//class uses CH20_06LIST as a test driver
class Stage extends Commandlet;
//#1 Class attributes
var private Array<Actor> rgActors;
var private Array<Prop> rgProps;
var private Array<string> rgLines;
var string szLines;
var private Script stageScript;
var int iNumOfActors, iNumOfProps;
var bool fPlayStart;

//#2
//---------------------------- showBasicUse ------------------------
public function showBasicUse(){

  /*********************************************************
    Create an instance of the Stage object in a test file
    Stage firstStage;
    firstStage = new class'Stage';
    You can then either call this function:
    firstStage.showBasicUse();
    Or copy the code from this function into the test file and
    call each the interface with the object:
    firstStage.addActor();
   *********************************************************/
    //Orestes
    local string szLineIndent;
    szLineIndent = Chr(10) $ "         ";

    //Orestes-----------------------------------------------------
    addActor("Chorus", "Dressed in dark, with veils");
    addNewEvent("Somber", "Chorus",
              "O heap not misery on misery" $ szLineIndent $
              "As a fond mother I would plead this thee.",
               145);
    addProp(145, "Veils trailing",
                           "Raises hands to plea wiht Electra.");
    addActorInterpretation(145, "Chants and then silence");

    //play the line and the interpretation (here or elsewhere
    playScriptLine(145, 145, 145);
}//end showBasicUse

//#3
//--------------addActorInterpretation--------------------------
public function addActorInterpretation(int eleNum,
                                       string intepretation){
  local string nameOfActor, eleDirection;
  nameOfActor = stageScript.getElement(eleNum).getElementName();
  eleDirection = stageScript.getElement(eleNum).getDirection();
  getActor(nameOfActor).addInterpretation(eleDirection,
                                          intepretation, 
                                          eleNum);
}// end addActorInterpretation
//-------------- showActorInterpreatation--------------------------
public function showActorInterpreatation(int eleNum){
  local string nameOfActor, eleDirection, interpretation;
  nameOfActor = stageScript.getElement(eleNum).getElementName();
  eleDirection = stageScript.getElement(eleNum).getDirection();
  interpretation = getActor(nameOfActor).getInterpretation(
                                                  eleDirection,
                                                  eleNum);
  Log("              [" $  nameOfActor $ ":  " 
                        $ interpretation $ "]");
}//end showActorInterpreatation

//#4
//---------------------------- startScript-------------------------
function private startScript(){
   if(fPlayStart == false){
      stageScript = new class'Script';
      fPlayStart = true;
   }
}//end startScript
//#5
//--------------------------- addNewEvent-------------------------
function public addNewEvent(string direction,
                              string character,
                              string line,
                              int elementNumber ){
  local string element;
   //Initialize the script the first time
   startScript();
  //Append colons to element so they can be tokenized
  direction $= Chr(178);
  element $= direction;
  character $= Chr(178);
  element $= character;
  line $= Chr(178);
  element $= line;
  element $= string(elementNumber);
  stageScript.addElement(makeScriptElement(element));
}//end addNewEvent

//#6
//----------------------- makeScriptElement------------------------
private function Element makeScriptElement(string itemsForScript){
  local Element firstElement;
  local int iCtr;
  iCtr = 0;
  firstElement = new class'Element';
  Split(itemsForScript,Chr(178), rgLines);
  if(rgLines.Length == 4){
     firstElement.setElement(rgLines[iCtr],          //direction
                             rgLines[iCtr+1],        //character
                             rgLines[iCtr+2],        //lines
                             int(rgLines[iCtr+3]));  //ID number
     }else{
     firstElement.setElementNum(0);
    }
  return firstElement;
}//end addNewEvent

//#7
//--------------------playScriptLine----------------------------
//Assemble all the pieces you want to put into the line
//at this point
public function playScriptLine(int line, 
                               optional int propNum,
                               optional int scriptLine){

    local string szSpeech, szCharacter, szDirection;
    local int iNumber;
    //Use this to get characteristics of an actor.
    szCharacter = stageScript.getElement(line).getElementName();
    iNumber = stageScript.getElement(line).getElementNum();
    szSpeech = stageScript.getElement(line).getSpeech();
    szDirection = stageScript.getElement(line).getDirection();
   //Element number
    Log("          ------------------------------------");
    Log("         " $ "[" $ iNumber       $ "]");
   //Prop use
    if(propNum > 0){
       Log( "                              ["
       $ getProp(propNum).getName()    $ "]");
       Log( "                              ["
       $ getProp(propNum).getAction()  $ "]");
     }
    //Direction and character
    Log("         " $ "[" $ szDirection   $ "]");
    Log("         " $ Caps(szCharacter) $ ":"  );
    Log("         " $ "[" 
                    $ getActor(szCharacter).getAppearance()  
                    $ "]");
    //Speech
    Log("         " $ szSpeech);
    
    if(propNum > 0){
            showActorInterpreatation(scriptLine);
    }
}//end playScriptLine
//#8
//-----------------------getProp---------------------------
function Prop getProp(int propNum){
  local int iCtr;
  //access the prop in the prop array and assign the
  //vector values
  local Prop localProp;
  localProp = new class'Prop';
  iNumOfProps = rgProps.Length;
  iCtr = 0;
  if(propNum > 0){
      while(iCtr <= iNumOfProps){
         if(rgProps[iCtr].getNumber() == propNum){
           localProp = rgProps[iCtr];
           break;
         }///end inner if
      iCtr++;
      }//end while
   }else{
      //Return dummy if client expects a Prop
      Log("   No prop with the number: " $ propNum);
      localProp.setNumber(0);
   }//end else
   return localProp;
}//end getProp

//#9
//-----------------------updatePropPos-------------------------
//Provide a number for the prop to be updated
//Then set the x, y, and z coordinates of the prop
function updatePropPos(int propNum, int x, int y, int z){
  local Vector locVec;
  local int iCtr;
  locVec.x = x;
  locVec.y = y;
  locVec.z = z;
  //access the prop in the prop array and assign the
  //vector values
  iNumOfProps = rgProps.Length;
  if(propNum > 0){
      while(iCtr <= iNumOfProps){
         if(rgProps[iCtr].getNumber() == propNum){
           rgProps[iCtr].setPos(locVec);
           break;
         }///end inner if
      iCtr++;
      }//end while
   }else{
      Log("   Position update. No prop with that number.");
   }//end else
}//end updateProp
//#10
//---------------------------addProp-------------------------------
function addProp(int propNumber, string propName,
                                 string propAction){
  local Prop localProp;
  localProp = new class'Prop';
  iNumOfProps = rgProps.Length;

  if(propNumber > 0){
     localProp.setNumber(propNumber);
     localProp.setName(propName);
     localProp.setAction(propAction);
     //Add the Prop to the array
     rgProps.Insert(iNumOfProps, 1);
     iNumOfProps = rgProps.Length;
     rgProps[iNumOfProps - 1] = localProp;
  }else{
     Log("Not a valid numbers. Set number greater than 0." );
  }
}//end addProp

//#11
//------------------------addActor--------------------------
function addActor(string actorName,  
                  string actorAppearance){
  local Actor localActor;
  localActor = new class'Actor';
  iNumOfActors = rgProps.Length;

  if(Len(actorName) > 0){
     localActor.setName(actorName);
     localActor.setAppearance(actorAppearance);
     //Add the Actor to the array
     rgActors.Insert(iNumOfActors, 1);
     iNumOfActors = rgProps.Length;
     rgActors[iNumOfActors] = localActor;
  }else{
     Log("You must provide the name of the actor" );
  }
}//end addActor


//#12
//-----------------------getActor---------------------------
function Actor getActor(string actorName){
  local int iCtr;
  //access the Actor int the actor array
  local Actor localActor;
  localActor = new class'Actor';
  iNumOfActors = rgActors.Length;
  iCtr = 0;
  if(Len(actorName) != 0){
      while(iCtr < iNumOfActors){
         if(rgActors[iCtr].getName() == actorName){
           localActor = rgActors[iCtr];
           break;
         }///end inner if
      iCtr++;
      }//end while
   }else{
      //Return a place holder
      Log("   No actor by this name");
      localActor.setName("None");
   }//end else
   return localActor;
}//end getProp
